<?php

defined('ABSPATH') OR exit; //prevent from direct access

/**
* WPGuards Malware tab class
*/
class WPGMalware {

    public $user_plan;
    public $scans;
    public $time;

    public function __construct() {
        global $WPGuards;

        $this->user_plan = get_option( 'wpguards_user_plan', 'basic' );
        $this->scans = get_option( 'wpg_scans', array() );
        $this->time = $this->get_next_scan_time();

        $this->check_scans();

    }

    /**
     * Displays main malware tab content
     * @return void
     */
    public function main_content() {

        if ($this->user_plan == 'basic') $this->manual_scan();
        else $this->display_scans();

    }

    /**
     * Displays sidebar for basic and premium user
     * @return void
     */
    public function sidebar() {

        if ($this->user_plan == 'basic') $this->basic_sidebar();
        else $this->premium_sidebar();

    }
    
    /**
     * Gets timestamp of next scan
     * @return void
     */
    public function get_next_scan_time() {
        global $WPGuards;

        // 15 minutes offset
        $offset = 900;
        $opt_timestamp = get_option('wpg_next_scan_timestamp');

        if ( $opt_timestamp == false || $opt_timestamp < time() ) { // there old timestamp so make a call

        	if ($opt_timestamp !== false)
        		$this->check_scans();

            $scan_time = $WPGuards->WPGConnection->get_next_scan_time();
            
            if (empty($scan_time)) {
            	
            	if ($opt_timestamp > time()) $timestamp = $opt_timestamp + $offset;
        		else $timestamp = time() + 3600 + $offset;

            } else {

            	$timestamp = $scan_time + $offset;

            }

            update_option('wpg_next_scan_timestamp', $timestamp);

        } else { // there is option

            $timestamp = get_option('wpg_next_scan_timestamp');

        }

        return $timestamp;

    }

    /**
     * Manual scan metabox
     * @return void
     */
    public function manual_scan() {
        ?>
        <div class="postbox scan">
            <h3><span><?php _e('Scan your site','wpguards'); ?></span></h3>
            <div class="inside">
                <a href="http://sitecheck.sucuri.net/scanner/?scan=<?php echo site_url(); ?>" target="_blank"><?php _e('SCAN NOW', 'wpguards'); ?></a>
            </div>
        </div>
        <?php
    }

    /**
     * Displays basic sidebar
     * @return void
     */
    public function basic_sidebar() {
        ?>
        <div class="postbox upgrade">
            <h3><span><?php _e('Upgrade','wpguards'); ?></span></h3>
            <div class="inside">
                <?php _e('Get now higher plan to automaticly scan your site every','wpguards'); ?> 

                <div id="plans">
                    <span class="tab-cen1">24</span><span class="tab-mo"><?php _e('hours','wpguards'); ?></span>
                    <span class="or">OR</span>
                    <span class="tab-cen2">72</span><span class="tab-mo"><?php _e('hours','wpguards'); ?></span>
                </div>

                <div id="upgrade">
                    <?php echo order_button(false, __('Upgrade now','wpguards')); ?>
                </div>
                
            </div>
        </div>
        <?php
    }

    /**
     * Displays premium sidebar
     * @return void
     */
    public function premium_sidebar() {
        ?>
        <div class="postbox next">
            <h3><span><?php _e('Next scan','wpguards'); ?></span></h3>
            <div class="inside">

                <?php
                echo '<div id="next_scan">';

                    if ($this->time - time() <= 0) {

                        _e('Now','wpguards');

                    } else {

                        /* translators: Context - In 3 days, 2 hour, 1 minute regarding when the next scan will be made */
                        _e('In','wpguards');

                        echo ' <acronym title="'.wpg_format_time($this->time).'">';
                            echo seconds2human( $this->time - time() );
                        echo '</acronym>';

                    }

                echo '</div>';
                ?>

            </div>
        </div>

        <div class="postbox informations">
            <h3><span><?php _e('Informations','wpguards'); ?></span></h3>
            <div class="inside">
                
                <?php _e('There are stored 10 last scans.','wpguards'); ?>
                
            </div>
        </div>

        <div class="postbox scan">
            <h3><span><?php _e('Manual scan','wpguards'); ?></span></h3>
            <div class="inside">
                <a href="http://sitecheck.sucuri.net/scanner/?scan=<?php echo site_url(); ?>" target="_blank"><?php _e('SCAN NOW', 'wpguards'); ?></a>
            </div>
        </div>
        <?php
    }

    /**
     * Displays scans table
     * @return void
     */
    public function display_scans() {
        global $WPGuards;

        if ( empty($this->scans) ) {
            
            echo '<div class="big-alert red">'.__('There is no scans available yet','wpguards').'</div>';
            echo '<div class="small-alert blue">';
                _e('Next scan will be made within','wpguards');
                echo ' <acronym title="'.wpg_format_time($this->time).'">';
                    echo seconds2human( $this->time - time() );
                echo '</acronym>';
            echo '</div>';
            return;
            
        }

        // wpg_pr(unserialize($this->scans[3]['content']));

        echo '<table class="wp-list-table widefat fixed scans" cellspacing="0">';

            $this->display_scans_header();

            $this->display_scans_content();

        echo '</table>';

    }

    /**
     * Displays scans table header
     * @return void
     */
    public function display_scans_header() {
        ?>
        <thead>
            <tr>
                <th scope="col" id="scan" class="manage-column column-scan sorted desc"><a><span><?php _e('Scan','wpguards'); ?></span><span class="sorting-indicator"></span></a></th>
                <th scope="col" id="system" class="manage-column column-systen"><a><?php _e('System','wpguards'); ?></a></th>
                <th scope="col" id="blacklisted" class="manage-column column-blacklisted"><a><?php _e('Blacklisted','wpguards'); ?></a></th>
                <th scope="col" id="malware" class="manage-column column-malware"><a><?php _e('Malware','wpguards'); ?></a></th>
                <th scope="col" id="malicious-javascript" class="manage-column column-malicious-javascript"><a><?php _e('Malicious javascript','wpguards'); ?></a></th>
                <th scope="col" id="malicious-iframes" class="manage-column column-malicious-iframes"><a><?php _e('Malicious iframes','wpguards'); ?></a></th>
                <th scope="col" id="suspicious-redirections" class="manage-column column-suspicious-redirections"><a><?php _e('Suspicious redirections','wpguards'); ?></a></th>
                <th scope="col" id="spam" class="manage-column column-spam"><a><?php _e('Spam','wpguards'); ?></a></th>
            </tr>
        </thead>
        <?
    }

    /**
     * Displays scans table content
     * @return void
     */
    public function display_scans_content() {
        ?>
        <tbody id="the-list" data-wp-lists="list:scans">
            <?php
            
            foreach ($this->scans as $id => $scan) {

                echo '<tr class="alternate scan" data-id="'.$id.'">';

                    echo '<td class="scan column-scan">';
                        echo wpg_format_time($scan['time']);
                        echo '<div class="row-actions">
                                <span class="view"><a href="#" class="scan-details" data-id="'.$id.'" title="'.__('View this scan','wpguards').'">'.__('Details','wpguards').'</a> | </span>
                                <span class="trash"><a href="#" class="scan-remove" data-noonce="'.wp_create_nonce('scan-remove').'" data-id="'.$id.'" title="'.__('Remove this scan','wpguards').'">'.__('Remove','wpguards').'</a></span>
                            </div>';
                    echo '</td>';
                    echo '<td class="system column-system">'.$this->get_indicator('WEBAPP', $scan).'</td>';
                    echo '<td class="blacklisted column-blacklisted">'.$this->get_indicator('BLACKLIST', $scan).'</td>';
                    echo '<td class="malware column-malware">'.$this->get_indicator('MALWARE', $scan).'</td>';
                    echo '<td class="malicious-javascript column-malicious-javascript">'.$this->get_indicator_subwarn('javascript', $scan['content']).'</td>';
                    echo '<td class="malicious-iframes column-malicious-iframes">'.$this->get_indicator_subwarn('iframe', $scan['content']).'</td>';
                    echo '<td class="suspicious-redirections column-suspicious-redirections">'.$this->get_indicator_subwarn('redirection', $scan['content']).'</td>';
                    echo '<td class="spam column-spam">'.$this->get_indicator_subwarn('spam', $scan['content']).'</td>';

                echo '</tr>';

                echo '<tr style="display: none;" class="details" data-id="'.$id.'">';
                    echo '<td colspan="7">';
                        $this->display_scan_details($id, unserialize($scan['content']));
                    echo '</td>';
                echo '</tr>';
                
            }
            ?>
        </tbody>
        <?
       
    }

    /**
     * Displays scan type indicator if there is a problem and when there isn't
     * @param  string $type scan type
     * @param  array $scan scan array
     * @return string       indicator icon
     */
    public function get_indicator($type, $scan) {

        $content = unserialize($scan['content']);
        
        if (isset($content[$type]['WARN'])) {
            return '<i class="icon-remove icon-2x red"></i>';
        } else{
            return '<i class="icon-ok icon-2x green"></i>';
        }
       
    }

    /**
     * Displays scan type indicator for subarray of WARN if there is a problem and when there isn't
     * @param  string $type scan type
     * @param  array $scan scan content array
     * @return string       indicator icon
     */
    public function get_indicator_subwarn($string, $scan) {

        $content = unserialize($scan);
        
        if ( !isset($content['MALWARE']['WARN']) )
            return '<i class="icon-ok icon-2x green"></i>';

        foreach ($content['MALWARE']['WARN'] as $warn) {
            
            if ( stripos($warn[1], $string) !== false )
                return '<i class="icon-remove icon-2x red"></i>';

        }
        
        return '<i class="icon-ok icon-2x green"></i>';
       
    }

    /**
     * Displays scan details row
     * @param  int $id   scan id
     * @param  array $scan scan
     * @return void
     */
    public function display_scan_details($id, $scan) {
        ?>

            <script type="text/javascript">jQuery(function() {
                jQuery("#tabs-<?php echo $id; ?>").tabs();
            });</script>

            <div id="tabs-<?php echo $id; ?>">

                <ul class="tabs-list">
                    <li><a href="#tabs-<?php echo $id; ?>-general"><?php _e('General','wpguards') ?></a></li>
                    <li><a href="#tabs-<?php echo $id; ?>-links"><?php _e('Links','wpguards') ?></a></li>
                    <li><a href="#tabs-<?php echo $id; ?>-blacklists"><?php _e('Blacklists','wpguards') ?></a></li>
                    <?php if (isset($scan['MALWARE'])): ?><li><a href="#tabs-<?php echo $id; ?>-malware"><?php _e('Malware','wpguards') ?></a></li><?php endif ?>
                </ul>

                <!-- tabs -->
                <div class="tabs-wrapper">

                    <div id="tabs-<?php echo $id; ?>-general" class="single-tab">
                        <?php $this->display_general_tab($scan); ?>
                    </div>

                    <?php if (isset($scan['LINKS'])): ?><div id="tabs-<?php echo $id; ?>-links" class="single-tab">
                        <?php $this->display_links_tab($scan); ?>
                    </div><?php endif ?>

                    <?php if (isset($scan['BLACKLIST'])): ?><div id="tabs-<?php echo $id; ?>-blacklists" class="single-tab">
                        <?php $this->display_blacklist_tab($scan); ?>
                    </div><?php endif ?>

                    <?php if (isset($scan['MALWARE'])): ?><div id="tabs-<?php echo $id; ?>-malware" class="single-tab">
                        <?php $this->display_malware_tab($scan); ?>
                    </div><?php endif ?>

                </div>

            </div>

        <?php
    }

    /**
     * Displays general scan details
     * @param  array $scan scan
     * @return void
     */
    public function display_general_tab($scan) {
        
        echo '<h4>'.__('Website info','wpguards').'</h4>';
            echo __('Site','wpguards').': '.$scan['SCAN']['SITE'][0].'<br />';
            echo __('Domain','wpguards').': '.$scan['SCAN']['DOMAIN'][0].'<br />';
            echo __('IP','wpguards').': '.$scan['SCAN']['IP'][0].'<br />';

        echo '<h4>'.__('System','wpguards').'</h4>';
            foreach ($scan['SYSTEM']['NOTICE'] as $notice) {
                echo $notice.'<br />';
            }

        if ( !isset($scan['WEBAPP']) )
            return;

        if ( isset($scan['WEBAPP']['INFO']) ) {

            echo '<h4>'.__('Application informations','wpguards').'</h4>';
                foreach ($scan['WEBAPP']['INFO'][0] as $notice) {
                    echo $notice.'<br />';
                }

                if ( isset($scan['WEBAPP']['NOTICE']) ) {
                    foreach ($scan['WEBAPP']['NOTICE'] as $notice) {
                        echo $notice.'<br />';
                    }
                }

        }

        if ( isset($scan['WEBAPP']['VERSION']) ) {

            echo '<h4>'.__('Application version','wpguards').'</h4>';
                foreach ($scan['WEBAPP']['INFO'][0] as $notice) {
                    echo $notice.'<br />';
                }

        }

        if ( isset($scan['WEBAPP']['WARN']) ) {

            echo '<h4 class="red">'.__('Application warning','wpguards').'</h4>';
                foreach ($scan['WEBAPP']['WARN'] as $notice) {
                    echo '<span class="red">'.$notice.'</span><br />';
                }

        }
       
    }

    /**
     * Displays links scan details
     * @param  array $scan scan
     * @return void
     */
    public function display_links_tab($scan) {
        
        if ( isset($scan['LINKS']['URL']) ) {

            echo '<h4>'.__('URLs found','wpguards').'</h4>';
                foreach ($scan['LINKS']['URL'] as $notice) {
                    echo $notice.'<br />';
                }

        }

        if ( isset($scan['LINKS']['JSLOCAL']) ) {

            echo '<h4>'.__('Javascript found','wpguards').'</h4>';
                foreach ($scan['LINKS']['JSLOCAL'] as $notice) {
                    echo $notice.'<br />';
                }

        }
       
    }

    /**
     * Displays blacklist scan details
     * @param  array $scan scan
     * @return void
     */
    public function display_blacklist_tab($scan) {
        
        if ( isset($scan['BLACKLIST']['INFO']) ) {

            echo '<h4>'.__('Blacklists','wpguards').'</h4>';
                foreach ($scan['BLACKLIST']['INFO'] as $notice) {
                    echo '<a href="'.$notice[1].'" target="_blank">'.$notice[0].'</a><br />';
                }

        }

        if ( isset($scan['BLACKLIST']['WARN']) ) {

            echo '<h4 class="red">'.__('Blacklisted','wpguards').'</h4>';
                foreach ($scan['BLACKLIST']['WARN'] as $notice) {
                    echo '<a href="'.$notice[1].'" class="red" target="_blank">'.$notice[0].'</a><br />';
                }

        }
       
    }

    /**
     * Displays malware scan details
     * @param  array $scan scan
     * @return void
     */
    public function display_malware_tab($scan) {
        
        if ( isset($scan['MALWARE']['WARN']) ) {

            echo '<h4 class="red">'.__('Malware','wpguards').'</h4>';
                foreach ($scan['MALWARE']['WARN'] as $notice) {
                    echo '<strong>'.$notice[0].'</strong><br />';
                    echo $notice[1].'<br /><br />';
                }

        }
       
    }

    /**
     * Check if there are new scans and if so - store them as an option
     * @return void
     */
    public function check_scans() {
        global $WPGuards;

        // Check if there is time to check new scans
        if ( $this->time > time() )
            return false;
        
        $scans = $WPGuards->WPGConnection->getScans();

        if (empty($scans)) return false;

        foreach ($scans as $scan) {

        	if ($scan->time == 0) continue;

            $this->scans[$scan->ID]['time'] = $scan->time;
            $this->scans[$scan->ID]['content'] = $scan->content;

        }

        krsort($this->scans);

        // keep last 10 scans
        $this->scans = array_slice($this->scans, 0, 10);

        update_option('wpg_scans', $this->scans);
       
    }

}